{-# LANGUAGE FlexibleContexts, RecordWildCards #-}

{-
  This module is part of Ironforge.
  Copyfree (f) 2014 Marvin Cohrs

  All wrongs revoked. Sharing is an act of love, not crime.
  Please share Ironforge with everyone you like.
  
  Redistribution and use in source and binary forms, with or without modification,
  are permitted provided that the following conditions are met:

      * Redistributions of source code must retain the above copyright notice,
        this list of conditions and the following disclaimer.
      * Redistributions in binary form must reproduce the above copyright notice,
        this list of conditions and the following disclaimer in the documentation
        and/or other materials provided with the distribution.
      * Neither the name of Marvin Cohrs nor the names of other
        contributors may be used to endorse or promote products derived
        from this software without specific prior written permission.
  
  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-}

module Game.Antisplice.Dungeon.Ironforge where

import Text.Chatty.Printer
import Text.Chatty.Scanner
import Text.Chatty.Expansion
import Text.Chatty.Expansion.Vars
import System.Chatty.Misc
import Text.Chatty.Extended.Printer
import Game.Antisplice
import Game.Antisplice.Dungeon.Ironforge.Cambridge
import Game.Antisplice.Dungeon.Ironforge.IronforgeCity
import Game.Antisplice.Dungeon.Ironforge.Kinds
import Game.Antisplice.Dungeon.Ironforge.School
import Control.Monad
import Control.Monad.Error.Class
import Data.Monoid

ironforge :: Constructor ()
ironforge = do
  kinds@Kinds{..} <- getKinds
  currencies@Currencies{..} <- getCurrencies
  methods@Methods{..} <- getMethods
  atoms@Atoms{..} <- getAtoms
  (greatForge, stationW, commonsW) <- ironforgeCity kinds currencies methods atoms
  stationC <- cambridge kinds currencies methods atoms
        
  -- Tram
  insertVocab "tram" Noun
  withRoom stationW $
    addRoomObject $ do
      setObjectTitle "the tram"
      setObjectDesc "This is the tram to Cambridge, Massachusetts."
      addObjectName "tram"
      addDescSeg "Enter the tram and ride to Cambridge, Massachusetts."
      onEnter $ do
        mprintLn "Immediately after you entered the tram, the overseer blows the whistle and the train departs."
        mprintLn "You have arrived at your destination: Cambridge, Massachusetts."
        enterAndAnnounce stationC
  withRoom stationC $
    addRoomObject $ do
      setObjectTitle "the tram"
      setObjectDesc "This is the tram to Ironforge, Dun Morogh."
      addObjectName "tram"
      addDescSeg "Enter the tram and ride to Ironforge, Dun Morogh."
      onEnter $ do
        mprintLn "The whistle is so loud that you almost get a tinnitus. The train begins to roll."
        mprintLn "You have arrived at your destination: Ironforge, Dun Morogh."
        enterAndAnnounce stationW
        
  -- The School
  (schoolStart,schoolEnd) <- school
  unipath greatForge schoolStart Down
  unipath schoolEnd greatForge Up
      
  -- Player
  steDef <- registerStereoM $ do
    mergeStereoM defaultStereo
    mergeStereoM visualStereo
    mergeStereoM manualStereo
    mergeStereoM consumeStereo
    mergeSkillM $ skill "oink" !+ EnsureLineEnd :-: Nil #->> mprintLn "Oink!" !+ implyGlobalCooldownA !+> implyCooldownA 5000
    mergeSkill $ skill "bake" !+ callRecipe byCooking
  subscribePlayer $ do
    setPlayerRoom greatForge
    setStatM Strength 6
    setStatM Agility 3
    setStatM Stamina 5
    setStatM Intelligence 1
    setStatM Spirit 2
    setStatM Armor 0
    setStatM AttackPower 5
    setStatM Haste 5
    setStatM CooldownDuration 1000
    addStereo steDef
    modifyCurrency money (+2000)
  mputv "prompt" $ Literal "${user} #{health}H \\$#{money.100}#?{ #{capital!otitle} #{ohealth}H}"
